# -*- coding: utf-8 -*-
#%%
"""
Created on Tue Mar 20 17:52:00 2018

@author: quint
"""
print ('####################------- Information ------- ####################')
print ('This script calculates the cross over length and depth of formation of interpreted stylolites using a fourier transform approach (Ebner et al., 2009)')
print ('variables to be changed are highligted with the following comment:  #-- Variable changed by user')

#-- This script calculates the cross over length and depth of formation of interpreted stylolites 
#-- For this script some varialbes need to be altered by the user 
#-- These variables are highlighted with the following comment #-- Variable changed by user

#-- This script has been devided into cells using: #%%
#-- Adviced to run the script slowly in parts so that errors are avoided

print ('This script has been devided into parts using: #%%') 
print ('Adviced to run the script slowly and in parts so that errors are avoided')
#%%
print ('####################------- Part 1 ------- ####################')
print ('')
print ('')
print ('loading functions and packages')
#-- Load in functions and packages needed to iterate through the script! 

import matplotlib.pyplot as plt
from scipy import interpolate
import pandas as pd
import numpy as np
from scipy import optimize 
import os
import csv

#-- This function bins the fourier transform data using a dynamic average approach 
def DataBinner(x, y, binsize):
    binstart = 0
    length_DataSet = len(x)
    numberofiterations = int(length_DataSet/binsize)
    binned_X = list(); binned_Y = list();
    for i in range(numberofiterations):
        xlist_I = list(); ylist_I = list();
        if binstart >= length_DataSet:
            break
        else:
            for j in range(binstart, binstart+binsize):
                xlist_I.append(x[j])
                ylist_I.append(y[j])
            Xmean = np.mean(xlist_I)
            Ymean = np.mean(ylist_I)
            binned_X.append(Xmean)
            binned_Y.append(Ymean)
            xlist_I *= 0
            ylist_I *= 0
            binstart = binstart + binsize
    return binned_X, binned_Y

#-- Powerlaw function for fitting the data
def PowerLaw_Function(x, a, n):
    return a * (x**-n)

#-- This function analytically solves the two power law functions (short and long wavenumbers)
def Solver(A_s, n_s, A_l, n_l):
    print ('A_s * x ^ n_s = A_l * x ^ n_l')
    print ('x = (A_s/A_l)^(1/(n_l-n_s))')
    Crossoverwavelength = np.power(A_s/A_l, 1/(n_l-n_s))
    return Crossoverwavelength

#-- Functions for calculating pressure of formation (Ebner et al., 2009 and Schmittbuhl et al., 2004)

def alpha(v):
    a = (1.0/3.0)
    b = (1.0+v)/(1.0-v)
    c = (1.0-2.0*v)/(1.0-v)
    alpha = a * b * c
    return alpha

def beta(v):
    beta = v*(1-2*v)/np.pi
    return beta

def Pressure_of_Stylolite_Formation(gamma, E, beta, alpha, Lc):
    sigma_zz = np.sqrt((((gamma*E)/beta)*(1/alpha))/Lc)
    return sigma_zz

print ('functions have been loaded')
#%%
print ('####################------- Part 2 ------- ####################')
print ('')
print ('')
print ('load, interpolate and plot the data set')
Filepath = os.path.normpath( "D:\PhD_FolderForAll_Results\Supplymentary_files_appendix_a\DATA" ) #-- Filepath changed by user
safefiles = True #-- Do you want to safe the files? 
_Name = 'Styolite_1_PointsV2' #-- Filename changed by user
excel_sheet = pd.read_csv(os.path.join( Filepath, '%s.csv'%(_Name)))

x = excel_sheet['X']
y = excel_sheet['Y']

f = interpolate.interp1d(x, y)
s_e =2
n = 1000 #-- Keep at thousand for now
xnew = np.linspace(np.min(x), np.max(x), n)
ynew = f(xnew)


plt.figure()
plt.plot(x,y)
ax = plt.gca()
ax.set_aspect('equal')
plt.ylim(np.min(y)-0.025,np.max(y)+0.025)
plt.xlim(np.min(x)-0.025,np.max(x)+0.025)
plt.xlabel('x(m)')
plt.ylabel('y(m)')
if safefiles:
    plt.savefig(os.path.join( Filepath, "OriginalStyloliteData_%s.eps" %(_Name)))
    plt.savefig(os.path.join( Filepath, "OriginalStyloliteData_%s.png" %(_Name)), dpi = 300)
plt.show()

plt.figure()
plt.plot(xnew,ynew, c = 'r')
ax = plt.gca()
ax.set_aspect('equal')
plt.ylim(np.min(y)-0.025,np.max(y)+0.025)
plt.xlim(np.min(x)-0.025,np.max(x)+0.025)
plt.xlabel('x(m)')
plt.ylabel('y(m)')
if safefiles:
    plt.savefig(os.path.join( Filepath, "InterpolatedStyloliteData_%s.eps" %(_Name)))
    plt.savefig(os.path.join( Filepath, "InterpolatedStyloliteData_%s.png" %(_Name)), dpi = 300)
plt.show()
#%%
print ('####################------- Part 3 ------- ####################')
print ('')
print ('')
print ('Apply and plot the fourier transform on the dataset: ', _Name)
print ('Located in the following folder: ', Filepath)

#-- m to mm 
conversionfactor = 1000
xnew2 = xnew * conversionfactor
ynew2 = ynew * conversionfactor

#-- Apply the fourier transform
T = (np.max(xnew2) - np.min(xnew2))/n
ps = np.abs(np.fft.fft(ynew2))**2
freqs = np.fft.fftfreq(xnew2.size, T)

#-- Plot the power spectrum on log log scale
plt.figure()
freqs_for_exp = freqs[s_e:n//2]
ps_forexp = ps[s_e:n//2]
plt.plot(freqs[s_e:n//2], ps[s_e:n//2])
plt.yscale('log')
plt.xscale('log')
plt.xlabel('wavenumber k (1/mm)')
plt.ylabel('power spectrum P(k)')
plt.show()

#-- Plot the power spectrum on a normal scale
plt.figure()
plt.plot(freqs[s_e*10:n//2], ps[s_e*10:n//2])
plt.xlabel('wavenumber k (1/mm)')
plt.ylabel('power spectrum P(k)')
plt.show()

#-- Calculate the wavelength from the calculated wavenumbers
lengths = list()
for i in range(len(freqs[s_e*10:n//2])):
    l = 1/freqs[s_e*10:n//2][i]
    lengths.append(l)

#-- Plot the frequency vs wavelength 
plt.figure()
plt.plot(lengths, ps[s_e*10:n//2], c = 'k')
plt.xlabel('wavelength (mm)')
plt.ylabel('power spectrum P(k)')
plt.show()
#%%
print ('####################------- Part 4 ------- ####################')
print ('')
print ('')
print ('Bin and sort the data using the databinner function and sorter respectively')

binsize = 10 #-- Variable changed by user
skip_first_elements = 0 #-- Variable changed by user

print ('chosen bin size of the dynamic avarage approach: ', binsize)
print ('Number of first elements skipped: ', skip_first_elements)
print ('These variables can be changed by the user. They need to be defined seperately for each dataset')

#-- Bin the data
Binner = DataBinner(freqs_for_exp[skip_first_elements:], ps_forexp[skip_first_elements:], binsize)

#-- Plot the binned dataset
print ('plot the binned dataset')
plt.figure()
plt.gca()
plt.plot(freqs[s_e:n//2], ps[s_e:n//2], label = 'Power Spectrum')
plt.plot(Binner[0], Binner[1], c = 'r', label = 'Binned data')
plt.legend(loc = 'best')
plt.yscale('log')
plt.xscale('log')
plt.ylim(1e-1, 1e6)
plt.xlim(1e-2, 2e0)
plt.xlabel('wavenumber k (1/mm)')
plt.ylabel('power spectrum P(k)')
if safefiles:
    plt.savefig(os.path.join( Filepath, "FourierTransform_binned_%s.eps" %(_Name)))
    plt.savefig(os.path.join( Filepath, "FourierTransform_binned_%s.png" %(_Name)), dpi = 300)
plt.show()

#-- Sort wavelengths: This algorithm sort the wave numbers in long and short wave numbers (k). It uses the binned results 
spectrum_LIM = 0.0 #-- variable changed by user
longwavenumberlimit = 1.5 #-- Variable changed by user
shortwavenumberlimit = 0.27#-- Variable changed by user

print ('chosen long wavenumber limit: ', longwavenumberlimit)
print ('chosen short wavenumber limit: ', shortwavenumberlimit)
print ('This sorting will seperate the binned dataset in to two datasets which will be used to define two power laws for the long and short wavenumbers, respectively')
print ('These variables can be changed by the user. They need to be defined seperately for each dataset')

#-- Sorting loop
k_short = list(); pk_short = list(); k_long = list(); pk_long = list(); k_rest = list(); pk_rest = list();

for i in range(len(Binner[0])):
    if Binner[0][i] <= spectrum_LIM:
        print ('not used in analysis')
    elif Binner[0][i] <= shortwavenumberlimit:
        k_short.append(Binner[0][i])
        pk_short.append(Binner[1][i])
    elif 0.1 <= Binner[0][i] <= longwavenumberlimit:
        k_long.append(Binner[0][i])
        pk_long.append(Binner[1][i])
    elif Binner[0][i] >= longwavenumberlimit:
        k_rest.append(Binner[0][i])
        pk_rest.append(Binner[1][i])

# Use the powerlaw function to best fit the data. Seperate the short and the longer wavenumbers

print ('####################------- Part 5 ------- ####################')
print ('')
print ('')
print ('Curve fit (least error approach) the two power laws for the sorted data')

#-- Omptimized curve fit for the two data sets
print ('power law fit for the short wavenumbers (limit: ', shortwavenumberlimit, ')')
Params_ShortWavenumber, Params_ShortWavenumber_covariance = optimize.curve_fit(PowerLaw_Function, k_short, pk_short, p0 = [2,2]) 
print ('power law fit for the long wavenumbers (limit: ', longwavenumberlimit, ')')
Params_LongWavenumber, Params_LongWavenumber_covariance = optimize.curve_fit(PowerLaw_Function, k_long, pk_long, p0 = [2,2]) 

print ('')
print ('')
print ('Power law function parameters (A and n)', 'short wave number: ', Params_ShortWavenumber, 'long wave number: ', Params_LongWavenumber)


#-- Calculate the cross over wave length and cross over length of the two powerlaws
print ('')
print ('')
print ('Calculate the cross over wave length and cross over length of the two powerlaws')
A_s = Params_ShortWavenumber[0]
n_s = Params_ShortWavenumber[1]
print ('power for shortwavenumber is defined as: p(k) = ', A_s, '*k^-', n_s)
A_l = Params_LongWavenumber[0]
n_l = Params_LongWavenumber[1]
print ('power for longwavenumber is defined as: p(k) = ', A_l, '*k^-', n_l)

print('Solving for the cross-over length using the solver function')
Crossover_wavelength = Solver(A_s, -1*n_s, A_l, -1*n_l)
CrossoverLength = 1/Crossover_wavelength
print ('')
print ('')
print ('Crossover K: ', Crossover_wavelength, ' (1/mm)', ' Crossover Length: ', CrossoverLength, ' (mm)')

x_short = np.linspace(0.02, 1, 50)
x_long = np.linspace(0.05, 3, 50)
plt.figure()
#-- Plot all the results
print ('Plot all the results')
plt.gca()
plt.plot(x_short, PowerLaw_Function(x_short, Params_ShortWavenumber[0], Params_ShortWavenumber[1]), 'r--', label = '$P (k_l) = %sk^{-%s}$' %(int(Params_ShortWavenumber[0]), np.round(Params_ShortWavenumber[1], decimals = 2)))
plt.plot(x_long, PowerLaw_Function(x_long, Params_LongWavenumber[0], Params_LongWavenumber[1]), 'b--', label = '$P (k_s) = %sk^{-%s}$' %(int(Params_LongWavenumber[0]), np.round(Params_LongWavenumber[1], decimals = 2)))
plt.scatter(Crossover_wavelength, PowerLaw_Function(Crossover_wavelength, Params_ShortWavenumber[0], Params_ShortWavenumber[1]), c = 'green', marker = 's', s = 50,label = 'crossover length = %s (mm)' %(np.round(CrossoverLength, 2)))
plt.legend(loc = 'best', scatterpoints = 1)
plt.scatter(k_short, pk_short, c = 'r')
plt.scatter(k_long, pk_long, c = 'b')
plt.scatter(k_rest, pk_rest, c = 'grey')
plt.yscale('log')
plt.xscale('log')
plt.ylim(1e-1, 1e6)
plt.xlim(1e-2, 2e0)
plt.xlabel('wavenumber k (1/mm)')
plt.ylabel('power spectrum P(k)')
if safefiles:
    plt.savefig(os.path.join( Filepath, "binned_Powerlaw_%s.eps" %(_Name)))
    plt.savefig(os.path.join( Filepath, "binned_Powerlaw_%s.png" %(_Name)), dpi = 300)
plt.show()
#%%
print ('####################------- Part 6 ------- ####################')
print ('')
print ('')
print ('Calculate the depth of formation using the Ebner et al., 2009 paper')
print ('log all the used parameters and results to a csv file')
#-- Calculate the depth of formation using the Ebner et al., 2009 paper

#-- Parameters (surface tension: y, Youngs modulus: E, Poissons ratio: v, Rock density: rho )
y = 0.27 #-- (J m-2) #-- Variable changed by user
E = 30e9 #-- (Pa) #-- Variable changed by user
v = 0.25 #-- (-) #-- Variable changed by user
rho = 2500 #-- (kg m-3) #-- Variable changed by user
Lc = CrossoverLength/1000 #-- (m) #-- Calculated by the code

print ('input parameters: y =', y, '(J m-2)', 'E = ', E, 'Pa', 'v = ', v, 'rho = ', rho, 'kg m-3', 'cross over length = ',  Lc, 'm')

#-- Formulas sigma_zz = np.sqrt(((y*E)/beta*alpha)/Lc), alpha = (1/3)*(((1+v)/(1-v))*((1-2*v)/(1-v))) & beta = v*(1-2*v)*np.pi()

print ('#-- Formulas used: sigma_zz = np.sqrt(((y*E)/beta*alpha)/Lc), alpha = (1/3)*(((1+v)/(1-v))*((1-2*v)/(1-v))) & beta = v*(1-2*v)*np.pi()')
alpha = alpha(v)
beta = beta(v)

sigma_zz = Pressure_of_Stylolite_Formation(y, E, beta, alpha, Lc)

depth = sigma_zz/(rho*9.81)
print ('')
print ('')
print ('sigma zz: ', sigma_zz/1e6, ' (MPa)')
print ('depth (z): ', depth, ' (m)')

#-- write parameters to csv file 
print ('write the calculated data to a csv file')
File = open( os.path.normpath(os.path.join( Filepath , 'Parameters_Results_%s.csv' % (_Name)  ) ), 'w')
csvhandle = csv.writer(File, delimiter=',')
csvhandle.writerow(['Sample', _Name])
csvhandle.writerow(['binsize', binsize])
csvhandle.writerow(['Skipped elements', skip_first_elements])
csvhandle.writerow(['Start of the analysis at wavenumber: ', spectrum_LIM])
csvhandle.writerow(['Small wave number cut off', shortwavenumberlimit])
csvhandle.writerow(['large wave number cut off', longwavenumberlimit])
csvhandle.writerow(['Crossover wavenumber (1/mm)', Crossover_wavelength])
csvhandle.writerow(['Crossover wave length (mm)', CrossoverLength])
csvhandle.writerow(['surface tension (J m-2)', y])
csvhandle.writerow(['Youngs modulus (GPa)', E/1e9])
csvhandle.writerow(['Poissons ratio', v])
csvhandle.writerow(['rock density (kg m-3)', rho])
csvhandle.writerow(['Sigma zz (MPa)', sigma_zz/1e6])
csvhandle.writerow(['depth (m)', depth])
print ('script is done. We have calculated the cross over length, pressure and depth of formation using a fourier transform analysis. Used parameters and acquired results have been logged to the following file: Parameters_Results_%s.csv' % (_Name))

File.close()        
